<?php

namespace Drupal\webform_donate\Plugin\WebformElement;

use Drupal\currency\Entity\Currency;
use Drupal\Core\Form\FormStateInterface;
use Drupal\webform\Plugin\WebformElement\WebformCompositeBase;
use Drupal\webform\WebformSubmissionInterface;
use Drupal\media_entity\Entity\Media;

/**
 * Provides a 'webform_donate' element.
 *
 * @WebformElement(
 *   id = "webform_donate",
 *   label = @Translation("Donate"),
 *   category = @Translation("Webform Donate"),
 *   description = @Translation("Provides a form element to set the donation amount."),
 *   multiline = FALSE,
 *   composite = TRUE,
 *   states_wrapper = FALSE,
 * )
 */
class WebformDonate extends WebformCompositeBase {

  /**
   * {@inheritdoc}
   */
  protected function formatHtmlItemValue(array $element, WebformSubmissionInterface $webform_submission, array $options = []) {
    $value = $this->getValue($element, $webform_submission, $options);
    /** @var \Drupal\currency\Entity\CurrencyInterface $currency */
    $currency = Currency::load('GBP');
    $amount = $currency->getSign() . number_format(floatval($value['amount']), 2);

    $items = [];
    $frequency_value = $value['frequency'];
    if ($frequency_value == 'recurring') {
      $frequency_value = 'Monthly';
    }
    $items['donation_frequency'] = (string) $this->t('<em>Frequency:</em> @value', ['@value' => $frequency_value]);
    $items['donation_amount'] = (string) $this->t('<em>Amount:</em> @value', ['@value' => $amount]);

    return $items;
  }

  /**
   * {@inheritdoc}
   */
  protected function formatTextItemValue(array $element, WebformSubmissionInterface $webform_submission, array $options = []) {
    $value = $this->getValue($element, $webform_submission, $options);
    /** @var \Drupal\currency\Entity\CurrencyInterface $currency */
    $currency = Currency::load('GBP');
    $amount = $currency->getSign() . number_format(floatval($value['amount']), 2);

    $items = [];
    $items['donation_frequency'] = (string) $this->t('Frequency: @value', ['@value' => $value['frequency']]);
    $items['donation_amount'] = (string) $this->t('Amount: @value', ['@value' => $amount]);
    $items['donation_currency'] = (string) $this->t('Currency: @value', ['@value' => $value['currency']]);
    return $items;
  }

  /**
   * @return array
   */
  public function getDefaultProperties() {
    $frequencies = $this->getFrequencies();

    $defaults = [
      'default_approach_code' => '',
      'frequency_default' => '',
    ];

    $frequency_defaults = [];
    foreach ($frequencies as $frequency_id => $frequency) {
      $frequency_defaults[$frequency_id . '_enabled'] = '';
      $frequency_defaults[$frequency_id . '_option_label'] = $frequency;
      $frequency_defaults[$frequency_id . '_minimum_other'] = 1;

      for ($i = 0; $i < 3; $i++) {
        $frequency_defaults[$frequency_id . '_amount_' . $i] = '';
        $frequency_defaults[$frequency_id . '_benefit_' . $i] = '';
      }

      $frequency_defaults[$frequency_id . '_hero_image_enabled'] = FALSE;
      $frequency_defaults[$frequency_id . '_image_reference'] = '';
      $frequency_defaults[$frequency_id . '_image_style'] = '';
    }

    $defaults += $frequency_defaults;

    return parent::getDefaultProperties() + $defaults;
  }

  /**
   * @return array
   */
  public static function getFrequencies($with_hash = FALSE) {
    $hash = $with_hash ? "#" : "";

    $frequencies = [
      $hash . 'one_off' => 'One off',
      $hash . 'recurring' => 'Recurring',
    ];

    return $frequencies;
  }

  /**
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *
   * @return array
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $frequencies = $this->getFrequencies();

    $form['wrapper'] = [
      '#type' => 'fieldset',
      '#title' => 'Webform Donate Settings',
    ];

    $form['wrapper']['default_approach_code'] = [
      '#type' => 'textfield',
      '#title' => t('Default approach code'),
    ];

    $form['wrapper']['frequency_default'] = [
      '#type' => 'select',
      '#title' => t('Default frequency'),
      '#options' => $frequencies,
      '#description' => t('Select which of the frequencies is the default'),
      '#default_value' => $this->configuration['frequency_default'],
    ];

    foreach ($frequencies as $frequency_id => $frequency) {
      $form['wrapper'][$frequency_id] = [
        '#type' => 'details',
        '#title' => t('Frequency - @frequency', array('@frequency' => $frequency)),
        '#open' => FALSE,
      ];

      $form['wrapper'][$frequency_id][$frequency_id . '_enabled'] = [
        '#type' => 'checkbox',
        '#title' => t('Enabled'),
      ];

      $form['wrapper'][$frequency_id]['wrapper'] = [
        '#type' => 'fieldset',
      ];

      $form['wrapper'][$frequency_id]['wrapper'][$frequency_id . '_option_label'] = [
        '#type' => 'textfield',
        '#title' => t('Option label'),
      ];

      $form['wrapper'][$frequency_id]['wrapper'][$frequency_id . '_minimum_other'] = [
        '#type' => 'number',
        '#title' => t('Minimum other amount'),
      ];

      $form['wrapper'][$frequency_id]['wrapper'][$frequency_id . '_amounts'] = [
        '#type' => 'table',
        '#header' => [
          t('Amount'),
          t('Benefit'),
        ],
      ];

      $amount = 3;

      for ($i = 0; $i < $amount; $i++) {
        $form['wrapper'][$frequency_id]['wrapper'][$frequency_id . '_amounts'][$i] = [
          $frequency_id . '_amount_' . $i => [
            '#type' => 'textfield',
            '#size' => 5,
          ],
          $frequency_id . '_benefit_' . $i => [
            '#type' => 'textfield',
          ],
        ];
      }

      $media = NULL;
      if (!empty($this->configuration[$frequency_id]['hero_image']['image_reference'])) {
        $media = Media::load($this->configuration[$frequency_id]['hero_image']['image_reference']);
      }

      $form['wrapper'][$frequency_id]['wrapper'][$frequency_id . '_hero_image'] = [
        '#type' => 'details',
        '#title' => 'Hero image',
        '#open' => FALSE,
      ];

      $form['wrapper'][$frequency_id]['wrapper'][$frequency_id . '_hero_image'][$frequency_id . '_hero_image_enabled'] = [
        '#type' => 'checkbox',
        '#title' => t('Enabled'),
        '#default_value' => $this->configuration[$frequency_id]['hero_image']['enabled'],
        '#description' => t('Displays a hero image for one off payments in the donate widget form.'),
      ];
      $form['wrapper'][$frequency_id]['wrapper'][$frequency_id . '_hero_image'][$frequency_id . '_image_reference'] = [
        '#title' => t('Hero image'),
        '#type' => 'entity_autocomplete',
        '#target_type' => 'media',
        '#default_value' => $media,
        '#description' => t('Enter the name of the media image to display.'),
      ];
      $form['wrapper'][$frequency_id]['wrapper'][$frequency_id . '_hero_image'][$frequency_id . '_image_style'] = [
        '#title' => t('Image style'),
        '#type' => 'select',
        '#default_value' => $this->configuration[$frequency_id]['hero_image']['image_style'],
        '#options' => image_style_options(FALSE),
        '#description' => t('Select display style of media image.'),
      ];
    }

    return parent::buildConfigurationForm($form, $form_state);
  }

  /**
   * Placeholder method for getting frequency/amounts settings.
   *
   * @todo replace with perhaps a trait for donation configuration.
   *
   * @return array
   *   An array of amounts by frequency.
   */
  public static function getAmounts(array $frequencies, array $element) {
    $amounts_full = [];
    $currency = Currency::load('GBP');

    foreach ($frequencies as $frequency_id => $frequency) {
      if (!empty($element['#' . $frequency_id . '_enabled'])) {
        $amounts = [];

        for ($i = 0; $i < 6; $i++) {
          if (!empty($element['#' . $frequency_id . '_benefit_' . $i])) {
            $amounts[$element['#' . $frequency_id . '_amount_' . $i]] = [
              'benefit' => $element['#' . $frequency_id . '_benefit_' . $i],
              'label' => $currency->getSign() . $element['#' . $frequency_id . '_amount_' . $i],
            ];
          }
        }
        $amounts_full[$frequency_id] = [
          'label' => !empty($element['#' . $frequency_id . '_option_label']) ? $element['#' . $frequency_id . '_option_label'] : $frequency,
          'amounts' => $amounts,
          'minimum_other' => $element['#' . $frequency_id . '_minimum_other'],
          'allow_hero_image' => $element['#' . $frequency_id . '_hero_image_enabled'],
          'hero_image_reference' => $element['#' . $frequency_id . '_image_reference'],
          'hero_image_style' => $element['#' . $frequency_id . '_image_style'],
        ];
      }
    }
    return $amounts_full;
  }

  /**
   * Get defaults.
   *
   * @return array
   *   Array of defaults.
   */
  public static function getAmountDefaults(array $frequencies, array $element) {
    $amount_defaults = [];

    if (!empty($element['#frequency_default'])) {
      $amount_defaults['frequency_default'] = $element['#frequency_default'];
    }
    return $amount_defaults;
  }

}
